﻿/*** Name:      Settings.cs
 *** Created:   11/27/2008, by: Roulaux, R
 *** Modified:  12/12/2008, by: van Vliembergen, RWL
 *** Source:    Bevat applicatie brede instellingen. Deze kunnen worden opgeslagen.
 *** Copyright: (c) Gr2 Dev 2008
 */
using System;
using System.Collections.Generic;
using System.Text;
using System.IO;

namespace Etaris.Core
{
    /// <summary>
    /// Bevat applicatie brede instellingen. Deze kunnen worden opgeslagen.
    /// </summary>
    [Serializable()]
    public class Settings
    {
        /// <summary>
        /// Instantie van de huidig geladen instellingen.
        /// </summary>
        public static Settings CurrentSettings;
        /// <summary>
        /// Locatie van het laatst geladen configuratiebestand.
        /// </summary>
        public static String DefaultConfigurationFilePath = Path.GetFullPath("Configuration\\default_config.txt");
        /// <summary>
        /// Locatie van de standaard muziekbestanden.
        /// </summary>
        public static String DefaultMusicFilePath = Path.GetFullPath("Music\\tetris.mp3");
        /// <summary>
        /// Locatie van het configuratiebestand.
        /// </summary>
        public String ConfigurationFilePath;
        /// <summary>
        /// Boolean voor Fullscreen (<i>true</i>) of windowed (<i>false</i>).
        /// </summary>
        public bool Fullscreen = false;
        /// <summary>
        /// Standaard speed-up rate.
        /// </summary>
        public double DefaultSpeedUpRate = 10;
        /// <summary>
        /// Grootte van de blokken in pixels.
        /// </summary>
        public int BlockSize = 15;
        /// <summary>
        /// Hoeveelheid shading.
        /// </summary>
        public int ShadingSize = 6;
        /// <summary>
        /// De door de speler behaalde topscores en eventuele standaard scores.
        /// </summary>
        public HighScores HighScoreList = new HighScores();
        /// <summary>
        /// Lijst met standaard topscores.
        /// </summary>
        public HighScores HighScoreDef = new HighScores();
        /// <summary>
        /// Lijst met mogelijke muziekbestanden.
        /// </summary>
        public List<string> PlayList = new List<string>();
        /// <summary>
        /// Boolean om te bepalen of muziek moet spelen of niet. Als <i>true</i> dan speelt muziek.
        /// </summary>
        public bool PlayMusic = false;

        /// <summary>
        /// Het volledige pad van dit document.
        /// </summary>
        [NonSerialized]
        private string filePath;

        /// <summary>
        /// De minimale blokgrootte.
        /// </summary>
        public const int MinimalBlockSize = 15;
        /// <summary>
        /// De maximale blokgrootte.
        /// </summary>
        public const int MaximalBlockSize = 50;

        /// <summary>
        /// Slaat de instellingen op, op de locatie waar deze instellingen ingelezen zijn.
        /// </summary>
        public void SaveSettings()
        {
            syscore.XMLStoreBase<Settings> store = new syscore.XMLStoreBase<Settings>();
            store.Comment = "© Gr 2 Dev 2008";
            store.Path = filePath; //kijk in de map waar de .exe staat
            store.InnerData = this;

            store.SaveSettings();
        }

        /// <summary>
        /// Laadt de instellingen.
        /// </summary>
        /// <param name="path">Locatie van XML-bestand.</param>
        /// <returns>Een <i>Settings</i> object.</returns>
        public static Settings LoadSettings(string path)
        {
            //Init.
            syscore.XMLStoreBase<Settings> store = new syscore.XMLStoreBase<Settings>();
            store.Comment = "© Gr 2 Dev 2008";
            store.Path = Path.GetFullPath(path);

            CurrentSettings = store.LoadSettings();
            Settings s = CurrentSettings;

            s.filePath = store.Path;
            s.HighScoreList.ClearList(s.HighScoreDef, true);
            if (s.PlayList.Count == 0) s.PlayList.Add(DefaultMusicFilePath);
            if (s.BlockSize < MinimalBlockSize || s.BlockSize > MaximalBlockSize) s.BlockSize = MinimalBlockSize;
            if (s.ShadingSize * 2 > s.BlockSize) s.ShadingSize = s.BlockSize / 2;

            //Finish.
            return s;
        }
    }
}

