﻿/*** Name:      BaseBlock.cs
 *** Created:   12/4/2008, by: Hilkens, RGM
 *** Modified:  12/14/2008, by: Hilkens, RGM
 *** Source:    Definieert een basisblok.
 *** Copyright: (c) Gr2 Dev 2008
 */
using System;
using System.Collections.Generic;
using System.Text;

namespace Etaris.Fields
{
    /// <summary>
    /// Definieert een basisblok.
    /// </summary>
    public abstract class BaseBlock
    {
        /// <summary>
        /// X-positie tellend vanaf 0 (van links). De eenheid is een blokbreedte.
        /// </summary>
        int left;

        /// <summary>
        /// Y-positie tellend vanaf 0 (van boven). De eenheid is een blokhoogte.
        /// </summary>
        int top;

        #region "Properties"
        /// <summary>
        /// X-positie tellend vanaf 0 (van links). De eenheid is een blokbreedte.
        /// </summary>
        public int Left
        {
            set { left = value; }
            get { return left; }
        }

        /// <summary>
        /// Y-positie tellend vanaf 0 (van boven). De eenheid is een blokhoogte.
        /// </summary>
        public int Top
        {
            set { top = value; }
            get { return top; }
        }
        #endregion

        /// <summary>
        /// Constructor. Initialiseert de klasse.
        /// </summary>
        /// <param name="left">X-positie tellend vanaf 0 (van links). De eenheid is een blokbreedte.</param>
        /// <param name="top">Y-positie tellend vanaf 0 (van boven). De eenheid is een blokhoogte.</param>
        public BaseBlock(int left, int top)
        {
            this.Left = left;
            this.Top = top;
        }

        /// <summary>
        /// Controleert of een zijde <i>side</i> een rand van het speelstuk is.
        /// </summary>
        /// <typeparam name="T">Het type blokken waaruit het speelstuk bestaat.</typeparam>
        /// <param name="piece">Een basisspeelstuk.</param>
        /// <param name="side">Een zijde.</param>
        /// <returns><i>true</i> als Side een rand van het speelstuk is, anders <i>false</i>.</returns>
        internal virtual bool IsBorder<T>(BasePiece<T> piece, Sides side) where T : BaseBlock
        {
            //Init.
            int dx = 0, dy = 0;

            switch (side)
            {
                case Sides.Top:
                    dy = -1;
                    break;
                case Sides.Right:
                    dx = 1;
                    break;
                case Sides.Bottom:
                    dy = 1;
                    break;
                case Sides.Left:
                    dx = -1;
                    break;
            }

            //Doorloop alle blokken van het bijbehorende speelstuk.
            foreach (T block in piece.Blocks)
                if (block.Left == Left + dx && block.Top == Top + dy)
                    return false;

            //Finish.
            return true; //Geen blokken aan de zijde side gevonden.
        }
    }
}
