﻿/*** Name:      FrmMain.cs
 *** Created:   9/25/2008, by: Hilkens, RGM
 *** Modified:  12/16/2008, by: Hilkens, RGM
 *** Source:    hoofdvenster.
 *** Copyright: (c) Gr2 Dev 2008
 */
using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Drawing.Drawing2D;
using System.Text;
using System.Windows.Forms;
using System.Reflection;
using Etaris.Fields;
using Etaris.Core;
using sysui;

namespace Etaris.Gui
{
    /// <summary>
    /// Hoofdvenster.
    /// </summary>
    public partial class FrmMain : Form
    {
        Settings currentSettings;
        PlayingField topField;
        PlayingField bottomField;
        ConfigurationFile configuration;
        Game game;
        const int drawingBoxSeperation = 5;
        const int margin = 5;
        int tempBlockSize;
        bool gameConstructed = false;
        Graphics boxBottomPreviewGraphics;
        Graphics boxTopPreviewGraphics;
        PieceTemplate bottomPreviewTemplate;
        PieceTemplate topPreviewTemplate;
        int WindowedLeft = 0;
        int WindowedTop = 0;
        Size WindowedSize;

        /// <summary>
        /// Constructor. Initialiseert de klasse.
        /// </summary>
        public FrmMain()
        {
            InitializeComponent();
            this.Resize += new EventHandler(frmMain_Resize);
            this.Activated += new EventHandler(frmMain_Activated);
            this.pieceTimer.Tick += new EventHandler(pieceTimer_Tick);
            this.FormClosing += new FormClosingEventHandler(FrmMain_FormClosing);
            PositionControls(boxBottomPreview.Size, pnlMain.Size, false);
        }

        /// <summary>
        /// Sluit de applicatie volledig. Dit gebeurt alleen na toestemming van de gebruiker.
        /// </summary>
        private void FrmMain_FormClosing(object sender, FormClosingEventArgs e)
        {
            if (game == null || e.CloseReason == CloseReason.ApplicationExitCall) return; //Het welkomsvenster kan dit venster ook sluiten.
            game.Paused = true;
            if (MessageBox.Show("Are you sure you want to exit Etaris?", Text, MessageBoxButtons.YesNo, MessageBoxIcon.Question) == DialogResult.No)
                e.Cancel = true;
            game.Paused = false;
        }

        /// <summary>
        /// Opent het welkomstvenster als dit venster voor het eerst start.
        /// </summary>
        private void frmMain_Activated(object sender, EventArgs e)
        {
            //Init.
            InitializeFormForGame(true);
            this.Activated -= frmMain_Activated;
            Program.frmWelcome.StartPosition = FormStartPosition.CenterParent;
            Program.frmWelcome.ShowDialog(this);
        }

        /// <summary>
        /// Ververst het scherm 100% bij vergroten/verkleinen.
        /// </summary>
        private void frmMain_Resize(object sender, EventArgs e)
        {
            //Pas drawingBoxen aan.
            boxTop.Height = (pnlMain.Height - drawingBoxSeperation) / 2;
            boxBottom.Height = (pnlMain.Height - drawingBoxSeperation) / 2;

            //Teken venster opnieuw.
            Refresh();
        }

        /// <summary>
        /// Tekent de achtergrond.
        /// </summary>
        protected override void OnPaintBackground(PaintEventArgs e)
        {
            //Init.
            if (ClientRectangle.Width == 0 || ClientRectangle.Height == 0) return;
            Graphics g = e.Graphics;
            Rectangle rec = this.ClientRectangle;

            //Teken achtergrond.
            g.FillRectangle(new LinearGradientBrush(rec, Color.FromArgb(117, 150, 210), Color.White, (float)0), rec);
        }

        /// <summary>
        /// Pauzeert het spel en geeft het pauzevenster weer.
        /// </summary>
        private void PauseGame()
        {
            game.Paused = true;
            Program.frmPause.Pause(this);
        }

        /// <summary>
        /// Hervat het spel als het gepauzeerd was.
        /// </summary>
        internal void ResumeGame()
        {
            game.Paused = false;
        }

        /// <summary>
        /// Initialiseert dit venster inclusief alle controllers voor een nieuw of aangepast spel.
        /// Een spel is aangepast als de blokgroottes in Settings.CurrentSettings zijn aangepast. De functie
        /// die dit aanpast is zelf verantwoordelijk voor het aanroepen van <i>InitializeFormForGame</i>().
        /// </summary>
        internal void InitializeFormForGame(bool playMusic)
        {
            currentSettings = Settings.CurrentSettings;
            FullScreen(currentSettings.Fullscreen);

            if (playMusic) InitializeMusic();

            if (!gameConstructed) return; //Deze functie kan ook door FrmOptions aangeroepen worden!

            //Initialiseert het venster.
            LayoutForm(configuration, currentSettings.BlockSize, false);
            ConstructGraphics();
            PropagateBlockSizes();
            PublishImages();
        }

        /// <summary>
        /// Initialiseert de muziekspeler.
        /// </summary>
        private void InitializeMusic()
        {
            //Initialiseert de muziekspeler.
            if (Program.musicPlayer != null) Program.musicPlayer.Stop();
            if (currentSettings.PlayMusic) { Program.musicPlayer = new MusicPlayer(currentSettings.PlayList); PlayMusic(true); }
            mnuMusic.Enabled = currentSettings.PlayMusic;
        }

        /// <summary>
        /// Positioneert de controllers en stelt de grootte van controllers en venster in.
        /// </summary>
        /// <param name="config">Een configuratiebestand.</param>
        /// <param name="blockSize">Lengte/breedte van een blok.</param>
        /// <param name="testOnly">Geeft aan of de afmetingen alleen moeten worden gecontroleerd.</param>
        internal bool LayoutForm(ConfigurationFile config, int blockSize, bool testOnly)
        {
            //Init.
            int boxMargin = 2 * boxBottom.GetMargin();
            int maxWidth, maxHeight;
            Size sizePreview;
            Size sizePanel = new Size();
            tempBlockSize = blockSize;

            //Bereken de afmetingen van het paneel.
            sizePanel = new Size(boxMargin + config.PlayingFieldWidth * blockSize, boxMargin + config.PlayingFieldHeight * blockSize);
            sizePanel.Height += sizePanel.Height + drawingBoxSeperation;

            //Bereken nieuwe afmetingen.
            config.GetMaximumPieceDimensions(out maxWidth, out maxHeight);
            sizePreview = new Size(boxMargin + maxWidth * blockSize, boxMargin + maxHeight * blockSize);

            //Positioneert de controllers in dit venster.
            return PositionControls(sizePreview, sizePanel, testOnly);
        }

        /// <summary>
        /// Positioneert de controllers in dit venster.
        /// </summary>
        /// <param name="sizePreview">Grootte van de previewboxen.</param>
        /// <param name="sizePanel">Grootte van het hoofdpaneel.</param>
        /// <param name="testOnly">Geeft aan of de afmetingen alleen moeten worden gecontroleerd.</param>
        private bool PositionControls(Size sizePreview, Size sizePanel, bool testOnly)
        {
            //Init.
            Rectangle workingArea = Screen.FromControl(this).WorkingArea;
            Size wndSize = new Size(sizePanel.Width + sizePreview.Width + 3 * margin,
                                        pnlMain.Top + pnlContainer.Top + sizePanel.Height + margin); //pnlMain.Top veranderd nooit.

            //Controleer totale grootte van het venster. Merk op dat deze controle NIET altijd gedaan wordt.
            if (testOnly && (wndSize.Width > workingArea.Width || wndSize.Height > workingArea.Height)) return false;
            if (testOnly) return true;

            //Stel de nieuwe afmetingen in.
            pnlMain.Size = sizePanel;
            boxTopPreview.Size = sizePreview;
            boxBottomPreview.Size = sizePreview;

            //Positie van controllers instellen.
            boxTopPreview.Left = pnlMain.Width + margin;
            boxBottomPreview.Top = sizePanel.Height - boxBottomPreview.Height;
            boxBottomPreview.Left = boxTopPreview.Left;

            //Venstergrootte instellen.   
            WindowedSize = wndSize;
            if (!TopMost)
            {
                ClientSize = wndSize;
                pnlContainer.Left = margin;
            }
            else
            {
                frmMain_Resize(null, null);
                pnlContainer.Left = Width / 2 - pnlContainer.Width / 2;
            }
            return true;

        }

        /// <summary>
        /// Construeert de afbeeldingen en graphics voor alle tekenvelden.
        /// </summary>
        private void ConstructGraphics()
        {
            //Init.
            int height = configuration.PlayingFieldHeight;
            int width = configuration.PlayingFieldWidth;
            int maxPieceWidth, maxPieceHeight;
            int size = currentSettings.BlockSize;
            Matrix m = new Matrix();
            Graphics g;

            //Bepaal grootste speelstuk.
            configuration.GetMaximumPieceDimensions(out maxPieceWidth, out maxPieceHeight);

            //Maak Images
            boxBottom.Image = new Bitmap(width * size, height * size);
            boxTop.Image = new Bitmap(width * size, height * size);
            boxBottomPreview.Image = new Bitmap(maxPieceWidth * size, maxPieceHeight * size);
            boxTopPreview.Image = new Bitmap(maxPieceWidth * size, maxPieceHeight * size);

            //Maar Graphics objecten.
            g = Graphics.FromImage(boxBottom.Image);
            bottomField.GraphicsContext = g;

            m.Scale(-1, -1); m.Translate(width * size - 1, height * size - 1, MatrixOrder.Append);
            g = Graphics.FromImage(boxTop.Image);
            g.Transform = m;
            topField.GraphicsContext = g;

            boxBottomPreviewGraphics = Graphics.FromImage(boxBottomPreview.Image);

            m.Reset(); m.Scale(-1, -1); m.Translate(maxPieceWidth * size - 1, maxPieceHeight * size - 1, MatrixOrder.Append);
            boxTopPreviewGraphics = Graphics.FromImage(boxTopPreview.Image);
            boxTopPreviewGraphics.Transform = m;

            //Laad afbeeldingen.
            PublishImages();
        }

        /// <summary>
        /// Stelt in alle momenteel geladen speelstukken en/of speelstuktemplates de blokgroottes in.
        /// </summary>
        private void PropagateBlockSizes()
        {
            //Init. 
            int blockSize, shadingSize;
            blockSize = currentSettings.BlockSize;
            shadingSize = currentSettings.ShadingSize;

            //Stel in de speelvelden (en zichtbare speelstuktemplates) de nieuwe blokgrootte in.
            bottomField.BlockSize = blockSize; topField.BlockSize = blockSize;
            bottomField.ShadingSize = shadingSize; topField.ShadingSize = shadingSize;

            if (bottomPreviewTemplate != null) bottomPreviewTemplate.SetPaintingSizes(blockSize, shadingSize);
            if (topPreviewTemplate != null) topPreviewTemplate.SetPaintingSizes(blockSize, shadingSize);

            //Teken alles opnieuw.
            bottomField.Paint(); topField.Paint();

            if (bottomPreviewTemplate != null) bottomPreviewTemplate.Paint(boxBottomPreviewGraphics);
            if (topPreviewTemplate != null) topPreviewTemplate.Paint(boxTopPreviewGraphics);
        }

        /// <summary>
        /// Publiceert alle afbeeldingen.
        /// </summary>
        private void PublishImages()
        {
            boxTop.ScrollBars = false; boxTop.Publish();
            boxBottom.ScrollBars = false; boxBottom.Publish();
            boxTopPreview.ScrollBars = false; boxTopPreview.Publish();
            boxBottomPreview.ScrollBars = false; boxBottomPreview.Publish();
        }

        /// <summary>
        /// Start een spel.
        /// </summary>
        /// <param name="configurationFile">Een configuratiebestand voor het nieuwe spel.</param>
        /// <param name="level">Het gekozen niveau.</param>
        /// <param name="speedUpRate">De gekozen speed-up rate.</param>
        internal void StartGame(ConfigurationFile configurationFile, int level, double speedUpRate)
        {
            //Init.
            currentSettings = Settings.CurrentSettings;
            configuration = configurationFile;
            game = new Game(configuration, pieceTimer, level, speedUpRate, new EventHandler(InitializeGame));
            game.Paused = false;

            //Werk score bij.
            if (lblInfo.Width > 82 + 12 * Math.Floor(Math.Log10(1 + game.Score)))
                lblInfo.Text = "Score: " + game.Score + " points";
            else
                lblInfo.Text = game.Score + " points";
        }

        /// <summary>
        /// Verwerkt een tik van de timer. De positie van een speelstuk wordt indien nodig veranderd.
        /// </summary>
        private void pieceTimer_Tick(object sender, EventArgs e)
        {
            game.TimerTick();

            //Teken het speelveld.
            boxBottom.RefreshImage();
            boxTop.RefreshImage();

            //Werk score bij.
            if (lblInfo.Width > 82 + 12 * Math.Floor(Math.Log10(1 + game.Score)))
                lblInfo.Text = "Score: " + game.Score + " points";
            else
                lblInfo.Text = game.Score + " points";
        }

        /// <summary>
        /// Verwerkt gebruikersinvoer van het toetsenbord.
        /// </summary>
        private void FrmMain_KeyDown(object sender, KeyEventArgs e)
        {
            //Init.
            FieldTypes activeField = game.ActiveField;

            //Verwerkt gebruikersinvoer.
            if (e.KeyCode == Keys.Left) { if (activeField == FieldTypes.BottomField) game.ProcessKey(Keys.Left); else game.ProcessKey(Keys.Right); }
            else if (e.KeyCode == Keys.Right) { if (activeField == FieldTypes.BottomField) game.ProcessKey(Keys.Right); else game.ProcessKey(Keys.Left); }
            else if (e.KeyCode == Keys.Up && activeField == FieldTypes.BottomField) game.ProcessKey(Keys.Down);
            else if (e.KeyCode == Keys.Down && activeField == FieldTypes.TopField) game.ProcessKey(Keys.Down);
            else if ((e.KeyCode & Keys.ShiftKey) > 0) game.ProcessKey(Keys.ShiftKey);

            //Teken het speelveld.
            boxBottom.RefreshImage();
            boxTop.RefreshImage();
        }

        /// <summary>
        /// Stuurt aanslagen van speciale toetsen naar <i>FrmMain_KeyDown</i>().
        /// </summary>
        protected override bool ProcessCmdKey(ref Message msg, Keys keyData)
        {
            if (msg.WParam == (IntPtr)16 || msg.WParam == (IntPtr)82) FrmMain_KeyDown(null, new KeyEventArgs(Keys.ShiftKey));
            else if (msg.WParam == (IntPtr)27) PauseGame();
            else if (msg.WParam == (IntPtr)37) FrmMain_KeyDown(null, new KeyEventArgs(Keys.Left));
            else if (msg.WParam == (IntPtr)38) FrmMain_KeyDown(null, new KeyEventArgs(Keys.Up));
            else if (msg.WParam == (IntPtr)39) FrmMain_KeyDown(null, new KeyEventArgs(Keys.Right));
            else if (msg.WParam == (IntPtr)40) FrmMain_KeyDown(null, new KeyEventArgs(Keys.Down));
            else return false;
            return true;
        }

        /// <summary>
        /// Initialiseert de <i>Game</i> klasse. Dit is een gebeurtenis die door de <i>Game</i> klasse wordt aangeroepen.
        /// </summary>
        /// <param name="sender"><i>Game</i> klasse welke deze functie aanroept.</param>
        /// <param name="args">Wordt niet gebruikt.</param>
        private void InitializeGame(object sender, EventArgs args)
        {
            //Init.
            Game game = (Game)sender;
            topField = game.TopField;
            bottomField = game.BottomField;

            //Maak graphics en stel event handlers in.
            gameConstructed = true;
            InitializeFormForGame(false);
            ((Game)sender).PreviewNextPiece += new Game.PreviewNextPieceHandler(game_PreviewNextPiece);
            ((Game)sender).GameOver += new Game.GameOverHandler(game_GameOver);
        }

        /// <summary>
        /// Dit is een gebeurtenis om bovenliggende klassen te laten weten dat het spel game-over is.
        /// </summary>
        /// <param name="score">Eindscore.</param>
        private void game_GameOver(int score)
        {
            int count = currentSettings.HighScoreList.Count;
            MessageBox.Show("GAME OVER!", this.Text, MessageBoxButtons.OK, MessageBoxIcon.Information);
            if (count > 0)
            {
                //Er wordt van uit gegaan dat er altijd een maximaal aantal items in de lijst is.
                if (currentSettings.HighScoreList[count - 1].FinalScore < score)
                    Program.frmHighScores.AddNewScore(score, this);
                else
                    Program.frmHighScores.ShowScoreList(this, -1);
            }
            gameConstructed = false;
            Program.frmWelcome.ShowDialog(this);
        }

        /// <summary>
        /// Tekent een speelstuktemplate naar het venster.
        /// </summary>
        private void game_PreviewNextPiece(PieceTemplate template, FieldTypes fieldType)
        {
            //Init.
            if (fieldType == FieldTypes.BottomField) bottomPreviewTemplate = template; else topPreviewTemplate = template;
            Graphics g = (fieldType == FieldTypes.BottomField) ? boxBottomPreviewGraphics : boxTopPreviewGraphics;
            Region r = new Region();

            //wis Preview gedeelte.
            r.MakeInfinite();
            g.FillRegion(Brushes.White, r);

            //Teken template, speelstuktemplates worden (nog) niet gecentreerd!
            template.SetPaintingSizes(currentSettings.BlockSize, currentSettings.ShadingSize);
            template.Paint(g);
            if (fieldType == FieldTypes.BottomField) boxBottomPreview.RefreshImage(); else boxTopPreview.RefreshImage();
        }

        #region "Menu"
        /// <summary>
        /// Menu: pauzeert het spel.
        /// </summary>
        private void mnuPause_Click(object sender, EventArgs e)
        {
            PauseGame();
        }

        /// <summary>
        /// Menu: opent het hoofdvenster.
        /// </summary>
        private void mnuWelcome_Click(object sender, EventArgs e)
        {
            game.Paused = true;
            if (MessageBox.Show("Are you sure you want to end this game?", Text, MessageBoxButtons.YesNo, MessageBoxIcon.Question) == DialogResult.Yes)
                Program.frmWelcome.ShowDialog(this);
            else
                game.Paused = false;
        }

        /// <summary>
        /// Menu: speelt of stopt muziek. Deze functie roept <i>PlayMusic</i>() aan.
        /// </summary>
        private void mnuMusic_Click(object sender, EventArgs e)
        {
            PlayMusic(!mnuMusic.Checked);
        }

        /// <summary>
        /// Speelt of stopt muziek.
        /// </summary>
        /// <param name="play"><i>true</i> als muziek gespeelt moet worden.</param>
        /// <remarks>
        /// Als er geen muziekspeler is, dan voert deze functie niets uit.
        /// </remarks>
        private void PlayMusic(bool play)
        {
            if (Program.musicPlayer != null)
            {
                if (play)
                { Program.musicPlayer.Play(); mnuMusic.Text = "Music playing"; }
                else
                { Program.musicPlayer.Pause(); mnuMusic.Text = "Music paused"; }

                mnuMusic.Checked = play;
            }
        }

        /// <summary>
        /// Menu: sluit de applicatie.
        /// </summary>
        private void mnuExit_Click(object sender, EventArgs e)
        {
            this.Close();
        }

        /// <summary>
        /// Menu: geeft copyright/programma informatie.
        /// </summary>
        private void mnuAbout_Click(object sender, EventArgs e)
        {
            game.Paused = true;

            string names = "\tHaan de, S\n\tNg, HS\n\tPenterman, AGJ\n\tRoulaux, R\n\tVliembergen van, RWL\n\tHilkens, RGM";
            Assembly assem = Assembly.GetExecutingAssembly();
            AssemblyName assemName = new AssemblyName(assem.FullName);
            Version ver = assemName.Version;
            MessageBox.Show("Etaris, © Gr2 Dev 2008 / TU/e Eindhoven. This program may not be distributed or modified without permission of Gr2 Dev / TU/e Eindhoven."
                + "\nVersion " + ver.Major + "." + ver.Minor + "." + ver.Build + "."
                + "\n\nAuthors (Gr2 Dev):\n" + names,
                "Etaris", MessageBoxButtons.OK, MessageBoxIcon.Information);

            game.Paused = false;
        }

        /// <summary>
        /// Wisselt tussen volledig scherm en normale venstermode.
        /// </summary>
        /// <param name="fullScreen">Geeft aan of dit venster volledig schermvullend moet zijn.</param>
        private void FullScreen(bool fullScreen)
        {
            if (fullScreen == TopMost) return;
            if (!fullScreen)
            {
                FormBorderStyle = FormBorderStyle.FixedSingle;

                Left = WindowedLeft; Top = WindowedTop;

                ClientSize = WindowedSize;
            }
            else
            {
                WindowedLeft = Left; WindowedTop = Top;

                Left = 0; Top = 0;

                FormBorderStyle = FormBorderStyle.None;
                Width = Screen.FromControl(this).Bounds.Width;
                Height = Screen.FromControl(this).Bounds.Height;
            }

            //Finish.
            TopMost = fullScreen;
            PositionControls(boxBottomPreview.Size, pnlMain.Size, false);
        }
        #endregion
    }
}
