﻿/*** Name:      FrmOptions.cs
 *** Created:   9/25/2008, by: Hilkens, RGM
 *** Modified:  12/18/2008, by: Hilkens, RGM
 *** Source:    Instellingenvenster.
 *** Copyright: (c) Gr2 Dev 2008
 */
using System;
using System.IO;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using Etaris.Core;

namespace Etaris.Gui
{
    /// <summary>
    /// Instellingenvenster.
    /// </summary>
    public partial class FrmOptions : Form
    {
        private string tmpConfigurationFilePath;
        private bool tmpFullScreen;
        private bool tmpPlayMusic;
        private double tmpDefaultSpeedUpRate;
        private int tmpBlockSize;
        private List<string> tmpPlayList = new List<string>();
        private ConfigurationFile config;

        /// <summary>
        /// Constructor. Initialiseert de klasse.
        /// </summary>
        public FrmOptions()
        {
            InitializeComponent();
            cmbBlockSize.Validating += new CancelEventHandler(cmbBlockSize_Validating);
            browseDlg.InitialDirectory = Application.StartupPath;

            string mainDir = Application.StartupPath, subPath = "";

            if (Directory.Exists(mainDir + "\\" + "Configuration")) subPath = "\\" + "Configuration"; else subPath = "";
            browseDlg.InitialDirectory = mainDir + subPath;

            if (Directory.Exists(mainDir + "\\" + "Music")) subPath = "\\" + "Music"; else subPath = "";
            musicAddDlg.InitialDirectory = mainDir + subPath;
        }

        /// <summary>
        /// Zorgt ervoor dat de correcte instellingen worden weergegeven als het instellingenvenster geladen wordt.
        /// </summary>
        private void FrmOptions_Load(object sender, EventArgs e)
        {
            //Init.
            int index;

            // Currentsettings inladen
            Settings currentSettings = Settings.CurrentSettings;
            tmpDefaultSpeedUpRate = currentSettings.DefaultSpeedUpRate;
            tmpBlockSize = currentSettings.BlockSize;
            tmpFullScreen = currentSettings.Fullscreen;
            tmpPlayList = currentSettings.PlayList;
            tmpPlayMusic = currentSettings.PlayMusic;

            //Laad het configuratiebestand een keer in om het aantal speelstukken uit te lezen.
            try
            {
                config = ConfigurationFile.LoadConfigurationFile(currentSettings.ConfigurationFilePath);
                txtNumParts.Text = config.PieceTemplatesCount.ToString();
            }
            catch
            {
                try
                {
                    currentSettings.ConfigurationFilePath = Settings.DefaultConfigurationFilePath;
                    config = ConfigurationFile.LoadConfigurationFile(Settings.DefaultConfigurationFilePath);
                    txtNumParts.Text = config.PieceTemplatesCount.ToString();
                }
                catch
                {
                    txtNumParts.Text = "Unknown: configuration file could not be loaded";
                    currentSettings.ConfigurationFilePath = null;
                }
            }

            // Configuratiebestand pad
            tmpConfigurationFilePath = currentSettings.ConfigurationFilePath;
            if (tmpConfigurationFilePath == null)
                cmbLibraryPath.Text = "";
            else
            {
                if ((index = cmbLibraryPath.FindStringExact(tmpConfigurationFilePath)) == -1)
                    index = cmbLibraryPath.Items.Add(Settings.CurrentSettings.ConfigurationFilePath);
                cmbLibraryPath.SelectedIndex = index;
            }

            // Block size
            if ((index = cmbBlockSize.FindStringExact(tmpBlockSize.ToString())) == -1)
                index = cmbBlockSize.Items.Add(tmpBlockSize.ToString());
            cmbBlockSize.SelectedIndex = index;

            numSpeedUpRate.Value = (decimal)tmpDefaultSpeedUpRate;  // Default speedup rate
            chkFullScreen.Checked = tmpFullScreen;                  // Full screen
            chkPlayMusic.Checked = tmpPlayMusic;                    // Play music

            // Play list
            lstPlayList.Items.Clear();

            foreach (string music in tmpPlayList)
                if (lstPlayList.FindStringExact(music) == -1) lstPlayList.Items.Add(music);
        }

        /// <summary>
        /// Slaat de gekozen instellingen op en sluit venster.
        /// </summary>
        private void cmdOK_Click(object sender, EventArgs e)
        {
            Settings.CurrentSettings.ConfigurationFilePath = tmpConfigurationFilePath;
            Settings.CurrentSettings.Fullscreen = tmpFullScreen;
            Settings.CurrentSettings.PlayMusic = tmpPlayMusic;
            Settings.CurrentSettings.DefaultSpeedUpRate = tmpDefaultSpeedUpRate;
            Settings.CurrentSettings.BlockSize = tmpBlockSize;
            Settings.CurrentSettings.PlayList = tmpPlayList;

            Settings.CurrentSettings.SaveSettings();
            Settings.CurrentSettings.ShadingSize = (int)((6.0 / 15.0) * tmpBlockSize);
            Program.frmMain.InitializeFormForGame(true);

            Close();
        }

        /// <summary>
        /// Sluit het venster zonder de aangepaste instellingen op te slaan.
        /// </summary>
        private void cmdCancel_Click(object sender, EventArgs e)
        {
            Close();
        }

        /// <summary>
        /// Opent het topscorevenster.
        /// </summary>
        private void cmdOpenHighScoreList_Click(object sender, EventArgs e)
        {
            Program.frmHighScores.ShowScoreList(this, -1);
        }

        /// <summary>
        /// Vult de topscorelijst met dummy waarden.
        /// </summary>
        private void cmdClearHighScoreList_Click(object sender, EventArgs e)
        {
            if (MessageBox.Show("Are you sure you want to delete the highscores?", Text, MessageBoxButtons.YesNo, MessageBoxIcon.Question) == DialogResult.Yes)
                Settings.CurrentSettings.HighScoreList.ClearList(Settings.CurrentSettings.HighScoreDef, false);
        }

        /// <summary>
        /// Opent een venster waarin een configuratiebestand uitgezocht kan worden.
        /// </summary>
        private void cmdBrowse_Click(object sender, EventArgs e)
        {
            if (browseDlg.ShowDialog() == DialogResult.OK)
            {
                int index;

                // Voegt file path toe aan de combobox als deze nog niet erin zit.
                if ((index = cmbLibraryPath.FindStringExact(browseDlg.FileName)) == -1)
                {
                    index = cmbLibraryPath.Items.Add(browseDlg.FileName);
                }

                // Selecteert het gekozen file path.
                cmbLibraryPath.SelectedIndex = index;
            }
        }

        /// <summary>
        /// Laadt het geselecteerde configuratiebestand in.
        /// </summary>
        private void cmdLoad_Click(object sender, EventArgs e)
        {
            Settings.CurrentSettings.ConfigurationFilePath = tmpConfigurationFilePath;

            // Laadt configuratiebestand
            config = LoadConfigurationFile(tmpBlockSize, false);

            //Geef het aantal ingeladen stukken weer op het optiescherm:
            if (config != null) txtNumParts.Text = config.PieceTemplatesCount.ToString();
            else txtNumParts.Text = "Unknown: configuration file could not be loaded";
        }

        /// <summary>
        /// Slaat tijdelijk de waarde van <i>ConfigurationFilePath</i> op.
        /// </summary>
        private void cmbLibraryPath_SelectedIndexChanged(object sender, EventArgs e)
        {
            tmpConfigurationFilePath = cmbLibraryPath.SelectedItem.ToString();
        }

        /// <summary>
        /// Slaat tijdelijk de waarde van <i>FullScreen</i> op.
        /// </summary>
        private void chkFullScreen_CheckedChanged(object sender, EventArgs e)
        {
            if (chkFullScreen.Checked) tmpFullScreen = true;
            else tmpFullScreen = false;
        }

        /// <summary>
        /// Slaat tijdelijk de waarde van <i>PlayMusic</i> op.
        /// </summary>
        private void chkPlayMusic_CheckedChanged(object sender, EventArgs e)
        {
            if (chkPlayMusic.Checked) tmpPlayMusic = true;
            else tmpPlayMusic = false;
        }

        /// <summary>
        /// Slaat tijdelijk de nieuwe waarde van <i>BlockSize</i> op.
        /// </summary>
        private void cmbBlockSize_Validating(object sender, CancelEventArgs e)
        {
            if (!Int32.TryParse(cmbBlockSize.Text, out tmpBlockSize))
            {
                MessageBox.Show("Block size is incorrect. Please fill in a number.", Text, MessageBoxButtons.OK, MessageBoxIcon.Exclamation);
                e.Cancel = true;
            }
            else
            {
                if (tmpBlockSize < Settings.MinimalBlockSize || tmpBlockSize > Settings.MaximalBlockSize)
                {
                    MessageBox.Show(String.Format("Block size is incorrect. Block size may not be less than {0} and not be bigger than {1}.", Settings.MinimalBlockSize, Settings.MaximalBlockSize), Text, MessageBoxButtons.OK, MessageBoxIcon.Exclamation);
                    e.Cancel = true;
                }
            }
        }

        /// <summary>
        /// Slaat tijdelijk de waarde van <i>DefaultSpeedUpRate</i> op.
        /// </summary>
        private void numSpeedUpRate_ValueChanged(object sender, EventArgs e)
        {
            tmpDefaultSpeedUpRate = decimal.ToDouble(numSpeedUpRate.Value);
        }

        /// <summary>
        /// Voegt een muziekbestand toe aan de playlist.
        /// </summary>
        private void cmdMusicAdd_Click(object sender, EventArgs e)
        {
            if (musicAddDlg.ShowDialog() == DialogResult.OK)
            {
                if (lstPlayList.FindStringExact(musicAddDlg.FileName) == -1)
                {
                    lstPlayList.Items.Add(musicAddDlg.FileName);
                    tmpPlayList.Add(musicAddDlg.FileName);
                }
            }
        }

        /// <summary>
        /// Verwijdert een muziekbestand uit de playlist.
        /// </summary>
        private void cmdMusicRemove_Click(object sender, EventArgs e)
        {
            if (lstPlayList.SelectedIndex != -1)
            {
                tmpPlayList.Remove(lstPlayList.SelectedItem.ToString());
                lstPlayList.Items.Remove(lstPlayList.SelectedItem);
            }
        }

        /// <summary>
        /// Laadt het configuratiebestand.
        /// </summary>
        /// <param name="blockSize">Lengte/breedte van een blok.</param>
        /// <param name="extendedMessage">Geeft aan of een foutmelding moet verwijzen naar het optiemenu.</param>
        /// <returns>Een configuratiebestand.</returns>
        public ConfigurationFile LoadConfigurationFile(int blockSize, bool extendedMessage)
        {
            //Init.
            string msg = "";
            if (extendedMessage) msg = "\r\nPlease select a valid configuration file in the options menu. For now, the default configuration file will be loaded.";
            bool loadDefault = false;
            bool tooBig = false;

            //Laad het configuratiebestand.
            try
            {
                config = ConfigurationFile.LoadConfigurationFile(Settings.CurrentSettings.ConfigurationFilePath);
                if (!Program.frmMain.LayoutForm(config, Settings.MinimalBlockSize, true)) { tooBig = true; config = null; loadDefault = true; }
            }
            catch (Exception ex)
            {
                MessageBox.Show("The configuration file does not exist, or is invalid!\r\n" + ex.Message + msg, this.Text, MessageBoxButtons.OK, MessageBoxIcon.Exclamation);
                loadDefault = true;
                config = null;
            }

            //Laad het laatst succesvol geladen configuratiebestand indien dit nodig is.
            if (loadDefault)
            {
                try
                {
                    Settings.CurrentSettings.ConfigurationFilePath = Settings.DefaultConfigurationFilePath; //Herstel de settings, zodat aan UR-51 voldaan blijft worden!
                    config = ConfigurationFile.LoadConfigurationFile(Settings.DefaultConfigurationFilePath);
                    if (!Program.frmMain.LayoutForm(config, Settings.MinimalBlockSize, true)) { tooBig = true; config = null; } //Als het speelvenster te groot is, mag nooit verder geladen worden.
                }
                catch (Exception ex)
                {
                    MessageBox.Show("The default configuration file does not exist, or is invalid!\r\n" + ex.Message, this.Text, MessageBoxButtons.OK, MessageBoxIcon.Exclamation);
                    config = null;
                }
            }

            //Geeft een foutmelding indien het speelvenster te groot is.
            if (tooBig)
            {
                MessageBox.Show("The game window doesn’t fit your screen at the smallest block size!\n" +
                        "Use another configuration file with a smaller playing field.\n", Text,
                        MessageBoxButtons.OK, MessageBoxIcon.Exclamation);
            }

            //Mocht het volgende keer foutgaan, gebruik dan de laatst geladen configuratie.
            Settings.DefaultConfigurationFilePath = Settings.CurrentSettings.ConfigurationFilePath;

            //Laat alle speelstukken zien als het configuratiebestand met succes geladen is. Dit geldt niet vor het standaardbestand.
            if (!loadDefault && !extendedMessage && !tooBig) //Inladen gelukt, en niet door FrmWelcome aangeroepen
            {
                MessageBox.Show("Playingfield dimensions: " + config.PlayingFieldHeight.ToString() + " x " + config.PlayingFieldWidth.ToString(), this.Text, MessageBoxButtons.OK, MessageBoxIcon.Information);
                Program.frmConfigFile.DrawPreviews(config, this);
            }

            //Finish.
            return config;
        }
    }
}
