﻿/*** Name:      frmMain.cs
 *** Created:   10/20/2008, by: Hilkens, RGM
 *** Modified:  12/11/2008, by: Hilkens, RGM
 *** Source:    pauzevenster.
 *** Copyright: (c) Gr2 Dev 2008
 */
using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using System.Drawing.Drawing2D;

namespace Etaris.Gui
{
    /// <summary>
    /// Pauzevenster.
    /// </summary>
    public partial class FrmPause : Form
    {
        bool mayClose = false;

        /// <summary>
        /// Constructor. Initialiseert de klasse.
        /// </summary>
        public FrmPause()
        {
            InitializeComponent();
        }

        /// <summary>
        /// Verwerkt de Escape-toets.
        /// </summary>
        protected override bool ProcessCmdKey(ref Message msg, Keys keyData)
        {
            if (msg.WParam == (IntPtr)27) cmdResume.PerformClick();
            return base.ProcessCmdKey(ref msg, keyData);
        }

        /// <summary>
        /// Geeft het pauzevenster weer.
        /// </summary>
        /// <param name="owner">Eigenaar van het dialoogvenster.</param>
        internal void Pause(Form owner)
        {
            mayClose = false;
            ShowDialog(owner);
        }

        /// <summary>
        /// Sluit het pauzevenster en hervat het spel.
        /// </summary>
        private void cmdResume_Click(object sender, EventArgs e)
        {
            Program.frmMain.ResumeGame();
            mayClose = true;
            this.Close();
        }

        /// <summary>
        /// Geeft het optiesmenu weer.
        /// </summary>
        private void cmdOptions_Click(object sender, EventArgs e)
        {
            Program.frmOptions.ShowDialog(this);
            CenterToParent();
        }

        /// <summary>
        /// Sluit het spel af en geeft het hoofdvenster weer.
        /// </summary>
        private void cmdClose_Click(object sender, EventArgs e)
        {
            DialogResult reply = MessageBox.Show("Are you sure you want to end this game?", "Etaris", MessageBoxButtons.YesNo, MessageBoxIcon.Question);

            if (reply == DialogResult.Yes)
            {
                mayClose = true;
                Hide();
                Program.frmWelcome.ShowDialog(Program.frmMain);
            }
        }

        /// <summary>
        /// Tekent achtergrond.
        /// </summary>
        protected override void OnPaintBackground(PaintEventArgs e)
        {
            //Init.
            if (ClientRectangle.Width == 0 || ClientRectangle.Height == 0) return;
            Pen pen1 = new Pen(Color.Black, 1);
            Pen pen2 = new Pen(Color.Gray, 1);
            Graphics g = e.Graphics;
            Rectangle rec = ClientRectangle;
            LinearGradientBrush brush;

            //Maak brush.
            rec.X = 30; rec.Width -= 30;
            brush = new LinearGradientBrush(rec, Color.FromArgb(117, 150, 210), Color.White, (float)0);
            brush.WrapMode = WrapMode.TileFlipX;

            //Teken achtergrond.
            g.FillRectangle(brush, ClientRectangle);

            //Teken border.
            rec = ClientRectangle;
            rec.Width -= 1;
            rec.Height -= 1;
            g.DrawRectangle(pen1, rec);

            rec.X = 1; rec.Y = 1;
            rec.Width -= 2; rec.Height -= 2;
            g.DrawRectangle(pen2, rec);
        }

        /// <summary>
        /// Laadt het venster.
        /// </summary>
        private void frmPause_Load(object sender, EventArgs e)
        {
            pnlButtons.BackColor = Color.FromArgb(117, 150, 210);
        }

        /// <summary>
        /// Voorkomt dat dit venster sluit wanneer dit niet is toegestaan.
        /// </summary>
        private void FrmPause_FormClosing(object sender, FormClosingEventArgs e)
        {
            e.Cancel = !mayClose;
        }
    }
}
