﻿/*** Name:      FrmWelcome.cs
 *** Created:   9/25/2008, by: Hilkens, RGM
 *** Modified:  12/2/2008, by: Hilkens, RGM
 *** Source:    welkomsvenster.
 *** Copyright: (c) Gr2 Dev 2008
 */
using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using Etaris.Core;

namespace Etaris.Gui
{
    /// <summary>
    /// Welkomsvenster.
    /// </summary>
    public partial class FrmWelcome : Form
    {
        private bool loadNewGame = false;
        private double speedUpRate = -1;
        private int level = -1;

        /// <summary>
        /// Constructor. Initialiseert de klasse.
        /// </summary>
        public FrmWelcome()
        {
            InitializeComponent();
            this.FormClosing += new FormClosingEventHandler(frmWelcome_FormClosing);
            cmbLevel.SelectedIndex = 0;
        }

        /// <summary>
        /// Laadt het venster. De controllers worden ingesteld.
        /// </summary>
        private void frmWelcome_Load(object sender, EventArgs e)
        {
            udSpeedUpRate.Value = (decimal) Settings.CurrentSettings.DefaultSpeedUpRate; //Initieer udSpeedUpRate.
        }

        /// <summary>
        /// Sluit applicatie als het venster door de gebruiker gesloten wordt.
        /// </summary>
        private void frmWelcome_FormClosing(object sender, FormClosingEventArgs e)
        {
            //Voorkomt dat applicatie gesloten wordt als een nieuw spel gestart moet worden.
            if (loadNewGame)
                loadNewGame = false;
            else
            {
                if (MessageBox.Show("Are you sure you want to exit Etaris?", Text, MessageBoxButtons.YesNo, MessageBoxIcon.Question) == DialogResult.Yes)
                {
                    loadNewGame = true;
                    Application.Exit();
                }
                else
                    e.Cancel = true;
            }
                
        }

        /// <summary>
        /// Opent het topscorevenster.
        /// </summary>
        private void cmdHighScoreList_Click(object sender, EventArgs e)
        {
            Program.frmHighScores.ShowScoreList(this,-1);
        }

        /// <summary>
        /// Opent het instellingenvenster.
        /// </summary>
        private void cmdOptions_Click(object sender, EventArgs e)
        {
            Program.frmOptions.ShowDialog(this);
        }

        /// <summary>
        /// Sluit het programma.
        /// </summary>
        private void cmdExit_Click(object sender, EventArgs e)
        {
            this.Close();
        }

        /// <summary>
        /// Start een spel.
        /// </summary>
        private void cmdPlayGame_Click(object sender, EventArgs e)
        {
            //Laad het configuratiebestand.
            if (level == -1) { MessageBox.Show("Please choose a level.", Text, MessageBoxButtons.OK, MessageBoxIcon.Information); return; }
            ConfigurationFile config = Program.frmOptions.LoadConfigurationFile(Settings.CurrentSettings.BlockSize, true);
            if (config == null) return;

            this.Hide();
            Program.frmMain.StartGame(config, level, speedUpRate);
            loadNewGame = true; 
            this.Close();
        }

        /// <summary>
        /// Slaat de geselecteerde speedup-rate op.
        /// </summary>
        private void udSpeedUpRate_ValueChanged(object sender, EventArgs e)
        {
            speedUpRate = decimal.ToDouble(udSpeedUpRate.Value);
        }

        /// <summary>
        /// Slaat het geselecteerde niveau op.
        /// </summary>
        private void cmbLevel_SelectedIndexChanged(object sender, EventArgs e)
        {
            if (cmbLevel.SelectedIndex > 0)
            {
                level = Int32.Parse(cmbLevel.SelectedItem.ToString());
            }
        }
    }
}
